/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user.keycloak;

import static de.ozgcloud.user.keycloak.KeycloakUserRemoteService.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;

import jakarta.inject.Inject;

import org.bson.types.ObjectId;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.MethodOrderer;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.keycloak.admin.client.resource.RealmResource;

import de.ozgcloud.user.User;
import de.ozgcloud.user.UserTestFactory;
import io.quarkus.test.junit.QuarkusTest;
import io.quarkus.test.junit.TestProfile;
import io.quarkus.test.junit.mockito.InjectSpy;

@QuarkusTest
@TestProfile(KeycloakTestProfile.class)
class KeycloakUserRemoteServiceITCase {

	@InjectSpy
	KeycloakUserRemoteService service;
	@InjectSpy
	KeycloakApiService apiService;
	@Inject
	RealmResource realmResource;

	@Test
	void shouldCallGetAllUsers() {
		service.getAllUsers();

		verify(apiService).findAllUser();
	}

	@Nested
	@TestMethodOrder(MethodOrderer.OrderAnnotation.class)
	class TestUpdateOzgCloudUserId {

		private static final String KEYCLOAK_USER_ID = "90748555-7894-4854-8292-1f6106826962";

		@DisplayName("Set ozgCloudUserId attribute in keycloak on new user")
		@Test
		@Order(1)
		void shouldSetOzgCloudUserId() {
			User user = prepareUser();
			service.updateOzgCloudUserId(user);

			var ozgCloudUserId = getOzgCloudUserIdFromKeycloak();

			assertThat(user.getId().toString()).isEqualTo(ozgCloudUserId);
		}

		@DisplayName("Update ozgCloudUserId attribute in keycloak on existing user")
		@Test
		@Order(2)
		void shouldUpdateOzgCloudUserId() {
			var oldCloudUserId = getOzgCloudUserIdFromKeycloak();
			service.updateOzgCloudUserId(prepareUser());

			var newCloudUserId = getOzgCloudUserIdFromKeycloak();

			assertThat(oldCloudUserId).isNotEqualTo(newCloudUserId);
		}

		private User prepareUser() {
			return UserTestFactory.createBuilder()
			  .id(new ObjectId())
			  .externalId(KEYCLOAK_USER_ID)
			  .keycloakUserId(KEYCLOAK_USER_ID)
			  .email("dorothea@test.local")
			  .firstName("Dorothea")
			  .lastName("Doe")
			  .fullName("Dorothea Doe")
			  .fullNameReversed("Doe Dorothea")
			  .username("dorothea")
			  .organisationsEinheitIds(List.of("248240886", "9030229", "10363455"))
			  .build();
		}

		private String getOzgCloudUserIdFromKeycloak() {
			var userResource = realmResource.users().get(KEYCLOAK_USER_ID);
			return userResource.toRepresentation().getAttributes().get(ATTRIBUTE_NAME_OZG_CLOUD_USER_ID).getFirst();
		}

	}

}